@@ -8,19 +8,19 @@ RB_GENERATE(KeyNodes_, KeyNode_, entry, key_node_cmp);
 
 int key_node_cmp(const KeyNode * const kn1, const KeyNode * const kn2)
 {
-    const Key *k1 = kn1->key;
-    const Key *k2 = kn2->key;    
+    const Key * const k1 = kn1->key;
+    const Key * const k2 = kn2->key;
     int ret;
     
-    if (k1->len < k2->len) {
+    if (k1->len == k2->len) {
         ret = memcmp(k1->val, k2->val, k1->len);
-        if (ret == 0) {
-            ret = 1;
+    } else if (k1->len < k2->len) {
+        if ((ret = memcmp(k1->val, k2->val, k1->len)) == 0) {
+            ret = -1;
         }
     } else {
-        ret = memcmp(k1->val, k2->val, k2->len);        
-        if (ret == 0 && k1->len != k2->len) {
-            ret = -1;
+        if ((ret = memcmp(k1->val, k2->val, k2->len)) == 0) {
+            ret = 1;
         }
     }
     return ret;
@@ -36,9 +36,9 @@ int get_key_node_from_key(PanDB * const db, Key * const key,
 
     *key_node = NULL;
     found_key_node = RB_FIND(KeyNodes_, &db->key_nodes, &scanned_key_node);
-    if (found_key_node != NULL) {
+    if (found_key_node != NULL) {        
         *key_node = found_key_node;
-        return 0;
+        return 1;
     }
     if (create == 0) {
         return 0;
@@ -60,7 +60,7 @@ int get_key_node_from_key(PanDB * const db, Key * const key,
     }
     *key_node = new_key_node;
     
-    return 1;
+    return 2;
 }
 
 void free_key_node(PanDB * const db, KeyNode * const key_node)
