@@ -28,7 +28,7 @@
 
 #include <sys/types.h>
 
-#include "event-config.h"
+#include "event2/event-config.h"
 
 #ifdef _EVENT_HAVE_SYS_TIME_H
 #include <sys/time.h>
