@@ -65,7 +65,7 @@
 extern "C" {
 #endif
 
-#include <event-config.h>
+#include <event2/event-config.h>
 #include <stdarg.h>
 #ifdef _EVENT_HAVE_SYS_TYPES_H
 #include <sys/types.h>
