@@ -26,7 +26,7 @@
 
 #include <sys/types.h>
 
-#include "event-config.h"
+#include "event2/event-config.h"
 
 #ifdef WIN32
 #include <winsock2.h>
@@ -51,6 +51,7 @@
 #include "mm-internal.h"
 #include "util-internal.h"
 #include "log-internal.h"
+#include "evthread-internal.h"
 #ifdef WIN32
 #include "iocp-internal.h"
 #include "defer-internal.h"
@@ -60,15 +61,19 @@ struct evconnlistener_ops {
 	int (*enable)(struct evconnlistener *);
 	int (*disable)(struct evconnlistener *);
 	void (*destroy)(struct evconnlistener *);
+	void (*shutdown)(struct evconnlistener *);
 	evutil_socket_t (*getfd)(struct evconnlistener *);
 	struct event_base *(*getbase)(struct evconnlistener *);
 };
 
 struct evconnlistener {
 	const struct evconnlistener_ops *ops;
+	void *lock;
 	evconnlistener_cb cb;
+	evconnlistener_errorcb errorcb;
 	void *user_data;
 	unsigned flags;
+	int refcnt;
 };
 
 struct evconnlistener_event {
@@ -82,12 +87,16 @@ struct evconnlistener_iocp {
 	evutil_socket_t fd;
 	struct event_base *event_base;
 	struct event_iocp_port *port;
-	CRITICAL_SECTION lock;
-	int n_accepting;
+	short n_accepting;
+	unsigned shutting_down : 1;
+	unsigned enabled : 1;
 	struct accepting_socket **accepting;
 };
 #endif
 
+#define LOCK(listener) EVLOCK_LOCK((listener)->lock, 0)
+#define UNLOCK(listener) EVLOCK_UNLOCK((listener)->lock, 0)
+
 struct evconnlistener *
 evconnlistener_new_async(struct event_base *base,
     evconnlistener_cb cb, void *ptr, unsigned flags, int backlog,
@@ -99,10 +108,36 @@ static void event_listener_destroy(struct evconnlistener *);
 static evutil_socket_t event_listener_getfd(struct evconnlistener *);
 static struct event_base *event_listener_getbase(struct evconnlistener *);
 
+#if 0
+static void
+listener_incref_and_lock(struct evconnlistener *listener)
+{
+	LOCK(listener);
+	++listener->refcnt;
+}
+#endif
+
+static int
+listener_decref_and_unlock(struct evconnlistener *listener)
+{
+	int refcnt = --listener->refcnt;
+	if (refcnt == 0) {
+		listener->ops->destroy(listener);
+		UNLOCK(listener);
+		EVTHREAD_FREE_LOCK(listener->lock, EVTHREAD_LOCKTYPE_RECURSIVE);
+		mm_free(listener);
+		return 1;
+	} else {
+		UNLOCK(listener);
+		return 0;
+	}
+}
+
 static const struct evconnlistener_ops evconnlistener_event_ops = {
 	event_listener_enable,
 	event_listener_disable,
 	event_listener_destroy,
+	NULL, /* shutdown */
 	event_listener_getfd,
 	event_listener_getbase
 };
@@ -142,6 +177,11 @@ evconnlistener_new(struct event_base *base,
 	lev->base.cb = cb;
 	lev->base.user_data = ptr;
 	lev->base.flags = flags;
+	lev->base.refcnt = 1;
+
+	if (flags & LEV_OPT_THREADSAFE) {
+		EVTHREAD_ALLOC_LOCK(lev->base.lock, EVTHREAD_LOCKTYPE_RECURSIVE);
+	}
 
 	event_assign(&lev->listener, base, fd, EV_READ|EV_PERSIST,
 	    listener_read_cb, lev);
@@ -203,8 +243,12 @@ evconnlistener_new_bind(struct event_base *base, evconnlistener_cb cb,
 void
 evconnlistener_free(struct evconnlistener *lev)
 {
-	lev->ops->destroy(lev);
-	mm_free(lev);
+	LOCK(lev);
+	lev->cb = NULL;
+	lev->errorcb = NULL;
+	if (lev->ops->shutdown)
+		lev->ops->shutdown(lev);
+	listener_decref_and_unlock(lev);
 }
 
 static void
@@ -222,13 +266,21 @@ event_listener_destroy(struct evconnlistener *lev)
 int
 evconnlistener_enable(struct evconnlistener *lev)
 {
-	return lev->ops->enable(lev);
+	int r;
+	LOCK(lev);
+	r = lev->ops->enable(lev);
+	UNLOCK(lev);
+	return r;
 }
 
 int
 evconnlistener_disable(struct evconnlistener *lev)
 {
-	return lev->ops->disable(lev);
+	int r;
+	LOCK(lev);
+	r = lev->ops->disable(lev);
+	UNLOCK(lev);
+	return r;
 }
 
 static int
@@ -250,7 +302,11 @@ event_listener_disable(struct evconnlistener *lev)
 evutil_socket_t
 evconnlistener_get_fd(struct evconnlistener *lev)
 {
-	return lev->ops->getfd(lev);
+	evutil_socket_t fd;
+	LOCK(lev);
+	fd = lev->ops->getfd(lev);
+	UNLOCK(lev);
+	return fd;
 }
 
 static evutil_socket_t
@@ -264,7 +320,11 @@ event_listener_getfd(struct evconnlistener *lev)
 struct event_base *
 evconnlistener_get_base(struct evconnlistener *lev)
 {
-	return lev->ops->getbase(lev);
+	struct event_base *base;
+	LOCK(lev);
+	base = lev->ops->getbase(lev);
+	UNLOCK(lev);
+	return base;
 }
 
 static struct event_base *
@@ -275,15 +335,30 @@ event_listener_getbase(struct evconnlistener *lev)
 	return event_get_base(&lev_e->listener);
 }
 
+void evconnlistener_set_error_cb(struct evconnlistener *lev,
+    evconnlistener_errorcb errorcb)
+{
+	LOCK(lev);
+	lev->errorcb = errorcb;
+	UNLOCK(lev);
+}
+
 static void
 listener_read_cb(evutil_socket_t fd, short what, void *p)
 {
 	struct evconnlistener *lev = p;
 	int err;
+	evconnlistener_cb cb;
+	evconnlistener_errorcb errorcb;
+	void *user_data;
+	LOCK(lev);
 	while (1) {
 		struct sockaddr_storage ss;
+#ifdef WIN32
+		int socklen = sizeof(ss);
+#else
 		socklen_t socklen = sizeof(ss);
-
+#endif
 		evutil_socket_t new_fd = accept(fd, (struct sockaddr*)&ss, &socklen);
 		if (new_fd < 0)
 			break;
@@ -291,13 +366,40 @@ listener_read_cb(evutil_socket_t fd, short what, void *p)
 		if (!(lev->flags & LEV_OPT_LEAVE_SOCKETS_BLOCKING))
 			evutil_make_socket_nonblocking(new_fd);
 
-		lev->cb(lev, new_fd, (struct sockaddr*)&ss, (int)socklen,
-		    lev->user_data);
+		if (lev->cb == NULL) {
+			UNLOCK(lev);
+			return;
+		}
+		++lev->refcnt;
+		cb = lev->cb;
+		user_data = lev->user_data;
+		UNLOCK(lev);
+		cb(lev, new_fd, (struct sockaddr*)&ss, (int)socklen,
+		    user_data);
+		LOCK(lev);
+		if (lev->refcnt == 1) {
+			int freed = listener_decref_and_unlock(lev);
+			EVUTIL_ASSERT(freed);
+			return;
+		}
+		--lev->refcnt;
 	}
 	err = evutil_socket_geterror(fd);
-	if (EVUTIL_ERR_ACCEPT_RETRIABLE(err))
+	if (EVUTIL_ERR_ACCEPT_RETRIABLE(err)) {
+		UNLOCK(lev);
 		return;
-	event_sock_warn(fd, "Error from accept() call");
+	}
+	if (lev->errorcb != NULL) {
+		++lev->refcnt;
+		errorcb = lev->errorcb;
+		user_data = lev->user_data;
+		UNLOCK(lev);
+		errorcb(lev, user_data);
+		LOCK(lev);
+		listener_decref_and_unlock(lev);
+	} else {
+		event_sock_warn(fd, "Error from accept() call");
+	}
 }
 
 #ifdef WIN32
@@ -305,6 +407,7 @@ struct accepting_socket {
 	CRITICAL_SECTION lock;
 	struct event_overlapped overlapped;
 	SOCKET s;
+	int error;
 	struct deferred_cb deferred;
 	struct evconnlistener_iocp *lev;
 	ev_uint8_t buflen;
@@ -369,8 +472,15 @@ start_accepting(struct accepting_socket *as)
 	const struct win32_extension_fns *ext = event_get_win32_extension_fns();
 	DWORD pending = 0;
 	SOCKET s = socket(as->family, SOCK_STREAM, 0);
-	if (s == INVALID_SOCKET)
-		return -1;
+	int error = 0;
+
+	if (!as->lev->enabled)
+		return 0;
+
+	if (s == INVALID_SOCKET) {
+		error = WSAGetLastError();
+		goto report_err;
+	}
 
 	setsockopt(s, SOL_SOCKET, SO_UPDATE_ACCEPT_CONTEXT,
 	    (char *)&as->lev->fd, sizeof(&as->lev->fd));
@@ -391,14 +501,20 @@ start_accepting(struct accepting_socket *as)
 		/* Immediate success! */
 		accepted_socket_cb(&as->overlapped, 1, 0, 1);
 	} else {
-		int err = WSAGetLastError();
-		if (err != ERROR_IO_PENDING) {
-			event_warnx("AcceptEx: %s", evutil_socket_error_to_string(err));
-			return -1;
+		error = WSAGetLastError();
+		if (error != ERROR_IO_PENDING) {
+			goto report_err;
 		}
 	}
 
 	return 0;
+
+report_err:
+	as->error = error;
+	event_deferred_cb_schedule(
+		event_base_get_deferred_cb_queue(as->lev->event_base),
+		&as->deferred);
+	return 0;
 }
 
 static void
@@ -411,32 +527,63 @@ stop_accepting(struct accepting_socket *as)
 }
 
 static void
-accepted_socket_invoke_user_cb(struct deferred_cb *cb, void *arg)
+accepted_socket_invoke_user_cb(struct deferred_cb *dcb, void *arg)
 {
 	struct accepting_socket *as = arg;
 
 	struct sockaddr *sa_local=NULL, *sa_remote=NULL;
 	int socklen_local=0, socklen_remote=0;
 	const struct win32_extension_fns *ext = event_get_win32_extension_fns();
+	struct evconnlistener *lev = &as->lev->base;
+	evutil_socket_t sock=-1;
+	void *data;
+	evconnlistener_cb cb=NULL;
+	evconnlistener_errorcb errorcb=NULL;
+	int error;
 
 	EVUTIL_ASSERT(ext->GetAcceptExSockaddrs);
 
+	LOCK(lev);
 	EnterCriticalSection(&as->lock);
 	if (as->free_on_cb) {
 		free_and_unlock_accepting_socket(as);
+		listener_decref_and_unlock(lev);
 		return;
 	}
 
-	ext->GetAcceptExSockaddrs(
-		as->addrbuf, 0, as->buflen/2, as->buflen/2,
-		&sa_local, &socklen_local, &sa_remote, &socklen_remote);
+	++lev->refcnt;
 
-	as->lev->base.cb(&as->lev->base, as->s, sa_remote,
-	    socklen_remote, as->lev->base.user_data);
+	error = as->error;
+	if (error) {
+		as->error = 0;
+		errorcb = lev->errorcb;
+	} else {
+		ext->GetAcceptExSockaddrs(
+			as->addrbuf, 0, as->buflen/2, as->buflen/2,
+			&sa_local, &socklen_local, &sa_remote,
+			&socklen_remote);
+		sock = as->s;
+		cb = lev->cb;
+		as->s = INVALID_SOCKET;
+	}
+	data = lev->user_data;
 
-	as->s = INVALID_SOCKET;
+	LeaveCriticalSection(&as->lock);
+	UNLOCK(lev);
 
-	start_accepting(as); /* XXXX handle error */
+	if (errorcb) {
+		WSASetLastError(error);
+		errorcb(lev, data);
+	} else {
+		cb(lev, sock, sa_remote, socklen_remote, data);
+	}
+
+	LOCK(lev);
+	if (listener_decref_and_unlock(lev))
+		return;
+
+	EnterCriticalSection(&as->lock);
+	start_accepting(as);
 	LeaveCriticalSection(&as->lock);
 }
 
@@ -446,6 +593,7 @@ accepted_socket_cb(struct event_overlapped *o, ev_uintptr_t key, ev_ssize_t n, i
 	struct accepting_socket *as =
 	    EVUTIL_UPCAST(o, struct accepting_socket, overlapped);
 
+	LOCK(&as->lev->base);
 	EnterCriticalSection(&as->lock);
 	if (ok) {
 		/* XXXX Don't do this if some EV_MT flag is set. */
@@ -454,16 +602,32 @@ accepted_socket_cb(struct event_overlapped *o, ev_uintptr_t key, ev_ssize_t n, i
 			&as->deferred);
 		LeaveCriticalSection(&as->lock);
 	} else if (as->free_on_cb) {
+		struct evconnlistener *lev = &as->lev->base;
 		free_and_unlock_accepting_socket(as);
+		listener_decref_and_unlock(lev);
+		return;
 	} else if (as->s == INVALID_SOCKET) {
 		/* This is okay; we were disabled by iocp_listener_disable. */
 		LeaveCriticalSection(&as->lock);
 	} else {
 		/* Some error on accept that we couldn't actually handle. */
+		BOOL ok;
+		DWORD transfer = 0, flags=0;
 		event_sock_warn(as->s, "Unexpected error on AcceptEx");
+		ok = WSAGetOverlappedResult(as->s, &o->overlapped,
+		    &transfer, FALSE, &flags);
+		if (ok) {
+			/* well, that was confusing! */
+			as->error = 1;
+		} else {
+			as->error = WSAGetLastError();
+		}
+		event_deferred_cb_schedule(
+			event_base_get_deferred_cb_queue(as->lev->event_base),
+			&as->deferred);
 		LeaveCriticalSection(&as->lock);
-		/* XXXX recover better. */
 	}
+	UNLOCK(&as->lev->base);
 }
 
 static int
@@ -473,17 +637,18 @@ iocp_listener_enable(struct evconnlistener *lev)
 	struct evconnlistener_iocp *lev_iocp =
 	    EVUTIL_UPCAST(lev, struct evconnlistener_iocp, base);
 
-	EnterCriticalSection(&lev_iocp->lock);
+	LOCK(lev);
+	lev_iocp->enabled = 1;
 	for (i = 0; i < lev_iocp->n_accepting; ++i) {
 		struct accepting_socket *as = lev_iocp->accepting[i];
 		if (!as)
 			continue;
 		EnterCriticalSection(&as->lock);
 		if (!as->free_on_cb && as->s == INVALID_SOCKET)
-			start_accepting(as); /* XXXX handle error */
+			start_accepting(as);
 		LeaveCriticalSection(&as->lock);
 	}
-	LeaveCriticalSection(&lev_iocp->lock);
+	UNLOCK(lev);
 	return 0;
 }
 
@@ -494,7 +659,8 @@ iocp_listener_disable_impl(struct evconnlistener *lev, int shutdown)
 	struct evconnlistener_iocp *lev_iocp =
 	    EVUTIL_UPCAST(lev, struct evconnlistener_iocp, base);
 
-	EnterCriticalSection(&lev_iocp->lock);
+	LOCK(lev);
+	lev_iocp->enabled = 0;
 	for (i = 0; i < lev_iocp->n_accepting; ++i) {
 		struct accepting_socket *as = lev_iocp->accepting[i];
 		if (!as)
@@ -507,7 +673,7 @@ iocp_listener_disable_impl(struct evconnlistener *lev, int shutdown)
 		}
 		LeaveCriticalSection(&as->lock);
 	}
-	LeaveCriticalSection(&lev_iocp->lock);
+	UNLOCK(lev);
 	return 0;
 }
 
@@ -516,10 +682,18 @@ iocp_listener_disable(struct evconnlistener *lev)
 {
 	return iocp_listener_disable_impl(lev,0);
 }
+
 static void
 iocp_listener_destroy(struct evconnlistener *lev)
 {
-	iocp_listener_disable_impl(lev,1);
+	struct evconnlistener_iocp *lev_iocp =
+	    EVUTIL_UPCAST(lev, struct evconnlistener_iocp, base);
+
+	if (! lev_iocp->shutting_down) {
+		lev_iocp->shutting_down = 1;
+		iocp_listener_disable_impl(lev,1);
+	}
+
 }
 
 static evutil_socket_t
@@ -541,6 +715,7 @@ static const struct evconnlistener_ops evconnlistener_iocp_ops = {
 	iocp_listener_enable,
 	iocp_listener_disable,
 	iocp_listener_destroy,
+	iocp_listener_destroy, /* shutdown */
 	iocp_listener_getfd,
 	iocp_listener_getbase
 };
@@ -558,6 +733,8 @@ evconnlistener_new_async(struct event_base *base,
 	struct evconnlistener_iocp *lev;
 	int i;
 
+	flags |= LEV_OPT_THREADSAFE;
+
 	if (!base || !event_base_get_iocp(base))
 		goto err;
 
@@ -573,7 +750,7 @@ evconnlistener_new_async(struct event_base *base,
 		event_sock_warn(fd, "getsockname");
 		goto err;
 	}
-	lev = mm_calloc(1, sizeof(struct evconnlistener_event));
+	lev = mm_calloc(1, sizeof(struct evconnlistener_iocp));
 	if (!lev) {
 		event_warn("calloc");
 		goto err;
@@ -582,15 +759,17 @@ evconnlistener_new_async(struct event_base *base,
 	lev->base.cb = cb;
 	lev->base.user_data = ptr;
 	lev->base.flags = flags;
+	lev->base.refcnt = 1;
 
 	lev->port = event_base_get_iocp(base);
 	lev->fd = fd;
 	lev->event_base = base;
+	lev->enabled = 1;
 
 	if (event_iocp_port_associate(lev->port, fd, 1) < 0)
 		goto err_free_lev;
 
-	InitializeCriticalSectionAndSpinCount(&lev->lock, 1000);
+	EVTHREAD_ALLOC_LOCK(lev->base.lock, EVTHREAD_LOCKTYPE_RECURSIVE);
 
 	lev->n_accepting = N_SOCKETS_PER_LISTENER;
 	lev->accepting = mm_calloc(lev->n_accepting,
@@ -611,6 +790,7 @@ evconnlistener_new_async(struct event_base *base,
 			free_and_unlock_accepting_socket(lev->accepting[i]);
 			goto err_free_accepting;
 		}
+		++lev->base.refcnt;
 	}
 
 	return &lev->base;
@@ -619,7 +799,7 @@ err_free_accepting:
 	mm_free(lev->accepting);
 	/* XXXX free the other elements. */
 err_delete_lock:
-	DeleteCriticalSection(&lev->lock);
+	EVTHREAD_FREE_LOCK(lev->base.lock, EVTHREAD_LOCKTYPE_RECURSIVE);
 err_free_lev:
 	mm_free(lev);
 err:
