@@ -30,7 +30,7 @@
 #include <windows.h>
 #endif
 
-#include "event-config.h"
+#include "event2/event-config.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -62,6 +62,7 @@
 #include "event2/buffer_compat.h"
 #include "event2/util.h"
 #include "event-internal.h"
+#include "evthread-internal.h"
 #include "util-internal.h"
 #include "log-internal.h"
 
@@ -181,7 +182,7 @@ multiple_write_cb(evutil_socket_t fd, short event, void *arg)
 	int len;
 
 	len = 128;
-	if (woff + len >= sizeof(wbuf))
+	if (woff + len >= (int)sizeof(wbuf))
 		len = sizeof(wbuf) - woff;
 
 	len = write(fd, wbuf + woff, len);
@@ -194,7 +195,7 @@ multiple_write_cb(evutil_socket_t fd, short event, void *arg)
 
 	woff += len;
 
-	if (woff >= sizeof(wbuf)) {
+	if (woff >= (int)sizeof(wbuf)) {
 		shutdown(fd, SHUT_WR);
 		if (usepersist)
 			event_del(ev);
@@ -470,6 +471,8 @@ end:
 		evutil_closesocket(pair2[1]);
 	if (rev)
 		event_free(rev);
+	if (wev)
+		event_free(wev);
 	if (closeev)
 		event_free(closeev);
 	if (base)
@@ -486,7 +489,7 @@ test_multiple(void)
 	/* Multiple read and write test */
 	setup_test("Multiple read/write: ");
 	memset(rbuf, 0, sizeof(rbuf));
-	for (i = 0; i < sizeof(wbuf); i++)
+	for (i = 0; i < (int)sizeof(wbuf); i++)
 		wbuf[i] = i;
 
 	roff = woff = 0;
@@ -515,7 +518,7 @@ test_persistent(void)
 	/* Multiple read and write test with persist */
 	setup_test("Persist read/write: ");
 	memset(rbuf, 0, sizeof(rbuf));
-	for (i = 0; i < sizeof(wbuf); i++)
+	for (i = 0; i < (int)sizeof(wbuf); i++)
 		wbuf[i] = i;
 
 	roff = woff = 0;
@@ -805,6 +808,9 @@ test_fork(void)
 
 	setup_test("After fork: ");
 
+	tt_assert(current_base);
+	evthread_make_base_notifiable(current_base);
+
 	write(pair[0], TEST1, strlen(TEST1)+1);
 
 	event_set(&ev, pair[1], EV_READ, simple_read_cb, &ev);
@@ -863,6 +869,7 @@ test_fork(void)
 
 	evsignal_del(&sig_ev);
 
+	end:
 	cleanup_test();
 }
 
@@ -1061,11 +1068,16 @@ test_signal_assert(void)
 	evsignal_del(&ev);
 
 	raise(SIGCONT);
+#if 0
 	/* only way to verify we were in evsig_handler() */
+	/* XXXX Now there's no longer a good way. */
 	if (base->sig.evsig_caught)
 		test_ok = 0;
 	else
 		test_ok = 1;
+#else
+	test_ok = 1;
+#endif
 
 	event_base_free(base);
 	cleanup_test();
@@ -1669,7 +1681,7 @@ evtag_fuzz(void *ptr)
 	evtag_init();
 
 	for (j = 0; j < 100; j++) {
-		for (i = 0; i < sizeof(buffer); i++)
+		for (i = 0; i < (int)sizeof(buffer); i++)
 			buffer[i] = rand();
 		evbuffer_drain(tmp, -1);
 		evbuffer_add(tmp, buffer, sizeof(buffer));
@@ -1875,7 +1887,7 @@ methodname_to_envvar(const char *mname, char *buf, size_t buflen)
 	char *cp;
 	evutil_snprintf(buf, buflen, "EVENT_NO%s", mname);
 	for (cp = buf; *cp; ++cp) {
-		*cp = toupper(*cp);
+		*cp = EVUTIL_TOUPPER(*cp);
 	}
 }
 #endif
@@ -2167,6 +2179,14 @@ end:
 #undef MANY
 }
 
+static void
+test_struct_event_size(void *arg)
+{
+	tt_int_op(event_get_struct_event_size(), <=, sizeof(struct event));
+end:
+	;
+}
+
 struct testcase_t main_testcases[] = {
 	/* Some converted-over tests */
 	{ "methods", test_methods, TT_FORK, NULL, NULL },
@@ -2212,6 +2232,8 @@ struct testcase_t main_testcases[] = {
 	{ "mm_functions", test_mm_functions, TT_FORK, NULL, NULL },
 	BASIC(many_events, TT_ISOLATED),
 
+	{ "struct_event_size", test_struct_event_size, 0, NULL, NULL },
+
 #ifndef WIN32
 	LEGACY(fork, TT_ISOLATED),
 #endif
@@ -2234,7 +2256,7 @@ struct testcase_t signal_testcases[] = {
 	LEGACY(immediatesignal, TT_ISOLATED),
 	LEGACY(signal_dealloc, TT_ISOLATED),
 	LEGACY(signal_pipeloss, TT_ISOLATED),
-	LEGACY(signal_switchbase, TT_ISOLATED),
+	LEGACY(signal_switchbase, TT_ISOLATED|TT_NO_LOGS),
 	LEGACY(signal_restore, TT_ISOLATED),
 	LEGACY(signal_assert, TT_ISOLATED),
 	LEGACY(signal_while_processing, TT_ISOLATED),
