@@ -495,6 +495,10 @@ static int find_near_context_cb(void *context_, void *entry,
     if (context->db->layer_type == LAYER_TYPE_SPHERICAL ||
         context->db->layer_type == LAYER_TYPE_GEOIDAL) {
         switch (context->db->accuracy) {
+        case ACCURACY_VINCENTY:
+            cd = vincenty_distance_between_geoidal_positions
+                (context->position, &scanned_slot->position);
+            break;
         case ACCURACY_HS:
             cd = hs_distance_between_geoidal_positions
                 (context->position, &scanned_slot->position);
@@ -507,8 +511,8 @@ static int find_near_context_cb(void *context_, void *entry,
             cd = gc_distance_between_geoidal_positions
                 (context->position, &scanned_slot->position);            
             break;
-        case ACCURACY_ROMBOID:
-            cd = romboid_distance_between_geoidal_positions
+        case ACCURACY_RHOMBOID:
+            cd = rhomboid_distance_between_geoidal_positions
                 (context->position, &scanned_slot->position);
             break;
         default:
@@ -789,7 +793,7 @@ static int find_in_rect_context_cb(void *context_, void *entry,
     }
     if (context->db->layer_type == LAYER_TYPE_SPHERICAL ||
         context->db->layer_type == LAYER_TYPE_GEOIDAL) {
-        cd = romboid_distance_between_geoidal_positions
+        cd = rhomboid_distance_between_geoidal_positions
             (context->position, &scanned_slot->position);
     } else {
         cd = distance_between_flat_positions(context->db,
@@ -1011,8 +1015,8 @@ int init_pan_db(PanDB * const db,
     pthread_rwlock_init(&db->rwlock_db, NULL);
     init_quad_node(&db->root);
     db->qbounds = (Rectangle2D) {
-        .edge0 = { .latitude = -180.0F, .longitude = -180.0F },
-        .edge1 = { .latitude =  180.0F, .longitude =  180.0F }
+        .edge0 = { .latitude = -90.0F, .longitude = -180.0F },
+        .edge1 = { .latitude =  90.0F, .longitude =  180.0F }
     };
     db->latitude_accuracy = db->longitude_accuracy =
         app_context.dimension_accuracy;
