@@ -6,6 +6,10 @@
 # define DEFAULT_BUFFERED_READ_BUFFER_SIZE ((size_t) 65536U)
 #endif
 
+#ifndef INITIAL_TRAVERSAL_STACK_SIZE 
+# define INITIAL_TRAVERSAL_STACK_SIZE ((size_t) 100U)
+#endif
+
 #define BINVAL_IS_EQUAL_TO_CONST_STRING(BV, S) \
     ((BV)->size == sizeof (S) - (size_t) 1U && \
         strncasecmp((BV)->val, (S), sizeof (S) - (size_t) 1U) == 0)
@@ -18,6 +22,11 @@ typedef struct BinVal_ {
     size_t max_size;
 } BinVal;
 
+typedef struct SBinVal_ {
+    char *val;
+    size_t size;
+} SBinVal;
+
 typedef struct BufferedReadContext_ {
     struct evbuffer *buf;
     int fd;
@@ -29,7 +38,9 @@ typedef struct BufferedReadContext_ {
 void skip_spaces(const char * * const str);
 
 int key_node_to_json(KeyNode * const key_node, yajl_gen json_gen,
-                     const _Bool with_properties);
+                     PanDB * const pan_db,                     
+                     const _Bool with_properties,
+                     const _Bool with_links);
 
 Position2D sin_projection(const Position2D * const position);
 Position2D flat_projection(const Position2D * const position);
@@ -44,6 +55,9 @@ Meters distance_between_flat_positions(const PanDB * const pan_db,
                                        const Position2D * const p1,
                                        const Position2D * const p2);
 
+Meters vincenty_distance_between_geoidal_positions(const Position2D * const p1,
+                                                   const Position2D * const p2);
+
 Meters hs_distance_between_geoidal_positions(const Position2D * const p1,
                                              const Position2D * const p2);
 
@@ -53,8 +67,8 @@ Meters gc_distance_between_geoidal_positions(const Position2D * const p1,
 Meters fast_distance_between_geoidal_positions(const Position2D * const p1,
                                                const Position2D * const p2);
 
-Meters romboid_distance_between_geoidal_positions(const Position2D * const p1,
-                                                  const Position2D * const p2);
+Meters rhomboid_distance_between_geoidal_positions(const Position2D * const p1,
+                                                   const Position2D * const p2);
 
 void untangle_rect(Rectangle2D * const rect);
 
